@@ -16,8 +16,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef NAUTILUS_BATCH_RENAME_UTILITIES_H
-#define NAUTILUS_BATCH_RENAME_UTILITIES_H
+#pragma once
 
 #include <gio/gio.h>
 #include <gtk/gtk.h>
@@ -68,6 +67,4 @@ void batch_rename_sort_lists_for_rename (GList    **selection,
                                          GList    **old_names,
                                          GList    **new_files,
                                          GList    **old_files,
-                                         gboolean   is_undo_redo);
-
-#endif /* NAUTILUS_BATCH_RENAME_UTILITIES_H */
+                                         gboolean   is_undo_redo);
\ No newline at end of file
