@@ -17,9 +17,9 @@
  *
  */
 
+#include <adwaita.h>
 #include <glib/gi18n.h>
 #include <gnome-autoar/gnome-autoar.h>
-#include <libadwaita-1/adwaita.h>
 
 #include <eel/eel-vfs-extensions.h>
 
@@ -564,7 +564,7 @@ nautilus_compress_dialog_controller_new (GtkWindow         *parent_window,
         gtk_editable_set_text (GTK_EDITABLE (name_entry), initial_name);
     }
 
-    gtk_widget_show (compress_dialog);
+    gtk_window_present (GTK_WINDOW (compress_dialog));
 
     return self;
 }
