@@ -143,12 +143,6 @@ get_launch_context (GtkWindow *parent_window)
 
     launch_context = gdk_display_get_app_launch_context (display);
 
-    if (parent_window != NULL)
-    {
-        gdk_app_launch_context_set_screen (launch_context,
-                                           gtk_window_get_screen (parent_window));
-    }
-
     return launch_context;
 }
 
@@ -237,7 +231,7 @@ nautilus_launch_application_by_uri (GAppInfo  *application,
 
 static void
 launch_application_from_command_internal (const gchar *full_command,
-                                          GdkScreen   *screen,
+                                          GdkDisplay  *display,
                                           gboolean     use_terminal)
 {
     GAppInfoCreateFlags flags;
@@ -251,21 +245,18 @@ launch_application_from_command_internal (const gchar *full_command,
     }
 
     app = g_app_info_create_from_commandline (full_command, NULL, flags, &error);
-    if (app != NULL && !(use_terminal && screen == NULL))
+    if (app != NULL && !(use_terminal && display == NULL))
     {
-        GdkDisplay *display;
         g_autoptr (GdkAppLaunchContext) context = NULL;
 
-        display = gdk_screen_get_display (screen);
         context = gdk_display_get_app_launch_context (display);
-        gdk_app_launch_context_set_screen (context, screen);
 
         g_app_info_launch (app, NULL, G_APP_LAUNCH_CONTEXT (context), &error);
     }
 
     if (error != NULL)
     {
-        g_message ("Could not start application: %s", error->message);
+        g_message ("Could not start app: %s", error->message);
     }
 }
 
@@ -280,7 +271,7 @@ launch_application_from_command_internal (const gchar *full_command,
  * @...: Passed as parameters to the application after quoting each of them.
  */
 void
-nautilus_launch_application_from_command (GdkScreen  *screen,
+nautilus_launch_application_from_command (GdkDisplay *display,
                                           const char *command_string,
                                           gboolean    use_terminal,
                                           ...)
@@ -306,7 +297,7 @@ nautilus_launch_application_from_command (GdkScreen  *screen,
 
     va_end (ap);
 
-    launch_application_from_command_internal (full_command, screen, use_terminal);
+    launch_application_from_command_internal (full_command, display, use_terminal);
 
     g_free (full_command);
 }
@@ -322,7 +313,7 @@ nautilus_launch_application_from_command (GdkScreen  *screen,
  * @parameters: Passed as parameters to the application after quoting each of them.
  */
 void
-nautilus_launch_application_from_command_array (GdkScreen          *screen,
+nautilus_launch_application_from_command_array (GdkDisplay         *display,
                                                 const char         *command_string,
                                                 gboolean            use_terminal,
                                                 const char * const *parameters)
@@ -346,14 +337,13 @@ nautilus_launch_application_from_command_array (GdkScreen          *screen,
         }
     }
 
-    launch_application_from_command_internal (full_command, screen, use_terminal);
+    launch_application_from_command_internal (full_command, display, use_terminal);
 
     g_free (full_command);
 }
 
 void
-nautilus_launch_desktop_file (GdkScreen   *screen,
-                              const char  *desktop_file_uri,
+nautilus_launch_desktop_file (const char  *desktop_file_uri,
                               const GList *parameter_uris,
                               GtkWindow   *parent_window)
 {
@@ -389,7 +379,7 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
     g_free (desktop_file_path);
     if (app_info == NULL)
     {
-        show_dialog (_("There was an error launching the application."),
+        show_dialog (_("There was an error launching the app."),
                      NULL,
                      parent_window,
                      GTK_MESSAGE_ERROR);
@@ -442,8 +432,6 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
     context = gdk_display_get_app_launch_context (gtk_widget_get_display (GTK_WIDGET (parent_window)));
     /* TODO: Ideally we should accept a timestamp here instead of using GDK_CURRENT_TIME */
     gdk_app_launch_context_set_timestamp (context, GDK_CURRENT_TIME);
-    gdk_app_launch_context_set_screen (context,
-                                       gtk_window_get_screen (parent_window));
     if (count == total)
     {
         /* All files are local, so we can use g_app_info_launch () with
@@ -466,7 +454,7 @@ nautilus_launch_desktop_file (GdkScreen   *screen,
     if (error != NULL)
     {
         message = g_strconcat (_("Details: "), error->message, NULL);
-        show_dialog (_("There was an error launching the application."),
+        show_dialog (_("There was an error launching the app."),
                      message,
                      parent_window,
                      GTK_MESSAGE_ERROR);
