@@ -12,6 +12,7 @@ struct _NautilusViewItem
     guint icon_size;
     gboolean is_cut;
     gboolean drag_accept;
+    gboolean is_loading;
     NautilusFile *file;
     GtkWidget *item_ui;
 };
@@ -25,6 +26,7 @@ enum
     PROP_ICON_SIZE,
     PROP_IS_CUT,
     PROP_DRAG_ACCEPT,
+    PROP_IS_LOADING,
     N_PROPS
 };
 
@@ -92,6 +94,12 @@ nautilus_view_item_get_property (GObject    *object,
         }
         break;
 
+        case PROP_IS_LOADING:
+        {
+            g_value_set_boolean (value, self->is_loading);
+        }
+        break;
+
         default:
         {
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -133,6 +141,12 @@ nautilus_view_item_set_property (GObject      *object,
         }
         break;
 
+        case PROP_IS_LOADING:
+        {
+            self->is_loading = g_value_get_boolean (value);
+        }
+        break;
+
         default:
         {
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -173,7 +187,10 @@ nautilus_view_item_class_init (NautilusViewItemClass *klass)
                                                  "", "",
                                                  NAUTILUS_TYPE_FILE,
                                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
-
+    properties[PROP_IS_LOADING] = g_param_spec_boolean ("is-loading",
+                                                        "", "",
+                                                        FALSE,
+                                                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
     g_object_class_install_properties (object_class, N_PROPS, properties);
 
     signals[FILE_CHANGED] = g_signal_new ("file-changed",
@@ -230,6 +247,15 @@ nautilus_view_item_set_drag_accept (NautilusViewItem *self,
     g_object_set (self, "drag-accept", drag_accept, NULL);
 }
 
+void
+nautilus_view_item_set_loading (NautilusViewItem *self,
+                                gboolean          loading)
+{
+    g_return_if_fail (NAUTILUS_IS_VIEW_ITEM (self));
+
+    g_object_set (self, "is-loading", loading, NULL);
+}
+
 NautilusFile *
 nautilus_view_item_get_file (NautilusViewItem *self)
 {
