@@ -26,8 +26,8 @@
 
 #pragma once
 
+#include <adwaita.h>
 #include <gtk/gtk.h>
-#include <libadwaita-1/adwaita.h>
 
 #include "nautilus-types.h"
 
